/* eslint-disable no-console */
import { Component, OnInit, Output, EventEmitter, QueryList, ViewChildren, ViewChild } from '@angular/core';
import { MatTreeNestedDataSource } from '@angular/material/tree';
import { NestedTreeControl } from '@angular/cdk/tree';
import { ResourcesLoaderService } from 'app/services/resources-loader.service';
import { IResource } from 'app/services/i-resource';
import { ContextsLoaderService } from 'app/services/contexts-loader.service';
import { IContextNode } from 'app/services/i-context-node';
import { Observable, map, startWith } from 'rxjs';
import { AbstractControl, FormBuilder, FormControl, FormGroup, Validators } from '@angular/forms';
import { Clipboard } from '@angular/cdk/clipboard';

@Component({
  selector: 'jhi-rsc-tree',
  templateUrl: './rsc-tree.component.html',
  styleUrls: ['./rsc-tree.component.scss'],
  providers: [ResourcesLoaderService, ContextsLoaderService],
})
export class RscTreeComponent implements OnInit {
  nestedTreeControl = new NestedTreeControl<IResource>(node => node.children);
  nestedDataSource = new MatTreeNestedDataSource<IResource>();
  @Output() treeNode = new EventEmitter<IResource>(); //emitting event to parent
  contexts: IContextNode[];
  public filteredContexts: Observable<IContextNode[]> | undefined;
  chooseContextForm: FormGroup | any;

  constructor(
    private fb: FormBuilder,
    private resLoader: ResourcesLoaderService,
    private ctxLoader: ContextsLoaderService,
    private clipboard: Clipboard
  ) {
    this.contexts = [];
    this.chooseContextForm = this.fb.group({
      namefield: ['', [Validators.required]],
      uidfield: [''],
    });
  }

  get namefield(): any {
    // eslint-disable-next-line @typescript-eslint/no-unsafe-return
    return this.chooseContextForm.get('namefield');
  }
  get uidfield(): any {
    // eslint-disable-next-line @typescript-eslint/no-unsafe-return
    return this.chooseContextForm.get('uidfield');
  }

  ngOnInit(): void {
    this.resLoader.getResourcesByContext().subscribe(res => {
      this.nestedDataSource.data = res;
    });

    // per la form dei contesti
    this.ctxLoader.getData().subscribe(res => {
      this.contexts = res;
      console.log('*******numero contesti...', res.length);
    });

    this.filteredContexts = this.chooseContextForm.get('namefield').valueChanges.pipe(
      startWith(''),
      map(ctx => (ctx ? this.filterContexts(ctx) : this.contexts.slice()))
    );
  } //fine oninit

  //TODO: questo metodo va inserito in una classe di utility common
  checkForErrorsIn(formControl: AbstractControl): string {
    if (formControl.hasError('required')) {
      return 'Value is required!';
    }
    return '';
  }

  //mettere ANY come tipo dell'argomento per evitare errore TypeScript nella map!!
  filterContexts(name: any): IContextNode[] {
    return this.contexts.filter(ctx => ctx.name.toLowerCase().includes(name.toLowerCase()));
  }

  displayFn(ctx: IContextNode): string {
    return ctx.name ? ctx.name + ' | ' + ctx.id : '';
  }

  onSelFn(ctx: IContextNode): void {
    console.log('-----------' + ctx.id);
    this.uidfield.value = ctx.id;
  }

  // (onSelectionChange)="onSelFn(ctx)"

  hasNestedChild(_: number, node: IResource): boolean {
    if (node.children == null) {
      return false;
    } else {
      return node.children.length > 0;
    }
  }

  copyUid(val: any): void {
    if (val instanceof FormControl) {
      this.clipboard.copy(val.getRawValue().id);
    } else {
      this.clipboard.copy('invalid value');
    }
  }
}
