/* eslint-disable no-console */

import {
  Component,
  Output,
  EventEmitter,
  OnInit,
  ViewChild,
  AfterViewInit,
  OnChanges,
  SimpleChanges,
  Input,
  QueryList,
} from '@angular/core';
import { IHostingnode } from 'app/services/i-hostinngnode';
import { MockCtxloaderService } from 'app/services/mock-ctxloader.service';
import { ResourcesLoaderService } from 'app/services/resources-loader.service';
import { MatTableModule, MatTableDataSource } from '@angular/material/table';
import { MatSort, MatSortModule } from '@angular/material/sort';
import { SharedModule } from 'app/shared/shared.module';
import { IContextNode } from 'app/services/i-context-node';
import { MatPaginator, MatPaginatorModule, PageEvent } from '@angular/material/paginator';
import { MatTab, MatTabGroup } from '@angular/material/tabs';
import { ITabbedEntity } from 'app/i-tabbed-entity';
import { RawjsonPaneComponent } from 'app/rawjson-pane/rawjson-pane.component';

@Component({
  standalone: true,
  selector: 'jhi-table-screen',
  templateUrl: './table-screen.component.html',
  styleUrls: ['./table-screen.component.scss'],
  providers: [MockCtxloaderService, ResourcesLoaderService],
  imports: [MatTableModule, MatSortModule, SharedModule, MatPaginatorModule, RawjsonPaneComponent],
})
export class TableScreenComponent implements OnInit, AfterViewInit, OnChanges {
  //TODO: questo a regime mettere sotto CONST
  displayedColumns: string[] = ['name', 'id', 'status', 'lastmod', 'memavailable', 'hdspace', 'actions'];
  dataFromService: IHostingnode[];
  dataSource = new MatTableDataSource();
  tableDetail: IHostingnode;

  @Input() myCtx: IContextNode; //fetching event from parent
  @Output() jsonEmitter = new EventEmitter<IHostingnode>();
  @ViewChild(MatSort) sort: MatSort;
  @ViewChild(MatPaginator) paginator: MatPaginator;

  //per tabbed view
  //@ViewChild(MatTabGroup)
  @ViewChild(MatTab)
  public tabGroup: MatTabGroup | any;
  public tabNodes: QueryList<MatTab> | any;
  public closedTabs = [];
  public tabs: ITabbedEntity[] = [{ title: 'JSON View', content: '', type: 0, id: '' }];
  selectedIdx = 0;
  chosenIds: string[] = [];
  ////////// fine tabbed view

  constructor(private myDataService: ResourcesLoaderService) {
    this.myCtx = {} as IContextNode;
    this.tableDetail = {} as IHostingnode;
    this.dataFromService = [];
    this.sort = new MatSort();
    this.paginator = {} as MatPaginator;
  }

  ngAfterViewInit(): void {
    this.dataSource.sort = this.sort;
    this.dataSource.paginator = this.paginator;
  }

  ngOnInit(): void {
    this.myDataService.getHostingNodes().subscribe(res => {
      this.dataFromService = res;
      this.dataSource.data = res;
    });
  }

  ngOnChanges(changes: SimpleChanges): void {
    //  console.log('---NELLA ONCHANGES...' + this.myCtx.name);
    for (const propName in changes) {
      if (propName === 'myCtx') {
        const param = changes[propName];
        this.myCtx = <IContextNode>param.currentValue;
        //controllo che l'oggetto non sia vuoto
        if (Object.keys(this.myCtx).length !== 0) {
          //mt qui va passato il parametro myCtx.id al rest
          this.myDataService.getHostingNodes().subscribe(res => {
            this.dataFromService = res;
            this.dataSource.data = res;
          });
        }
      }
    }
  }

  applyFilter(event: Event): void {
    const filterValue = (event.target as HTMLInputElement).value;
    this.dataSource.filter = filterValue.trim().toLowerCase();
  }

  // per tabbed pane (versione con aggiunta dinamica)

  removeTab(index: number): void {
    this.tabs.splice(index, 1);
  }

  addTab(itemId: string): void {
    if (!this.chosenIds.includes(itemId)) {
      const newItem = {
        id: itemId,
        title: itemId.substring(0, 20) + '...',
        //TODO: content a regime è la stringa JSON
        content: itemId,
        type: 0,
      };
      this.selectedIdx++;
      this.chosenIds.push(itemId);
      this.tabs.push(newItem);
    }
  }

  closeTab(index: number): void {
    console.debug('---------');
    console.debug(index);
    console.debug('---IDs:');
    console.debug(this.chosenIds);
    console.debug(this.tabs[index].id);
    console.debug('++++++++++');
    const x = this.chosenIds.indexOf(this.tabs[index].id);
    if (x !== -1) {
      this.chosenIds.splice(x, 1);
    }

    // this.closedTabs.push(index);
    this.tabGroup.selectedIndex = this.tabs.length - 1;
    //this.chosenIds.indexOf();
    this.tabs.splice(index, 1);
  }
}
